/*
 * DiceGame.cpp
 */

#include "DiceGame.h"
#include "Dice.h"
#include "DiceGamePlayer.h"
#include "DiceGameListener.h"

DiceGame::DiceGame(DiceGameListener* listener)
	: m_round(0), m_winner(0), m_listener(listener),
			m_player(new DiceGamePlayer()), m_dice(2)
{
	m_dice[0] = new Dice();
	m_dice[1] = new Dice();
}

DiceGame::~DiceGame()
{
	delete m_player;
	delete m_dice[0];
	delete m_dice[1];
}

void DiceGame::run()
{
	int result = 0;
	do {
		result = doRound();
	} while (result != DiceGameListener::STOP);
}

int DiceGame::doRound()
{
	int result = inquireOddOrEven();
	if (result == DiceGameListener::STOP) {
		return DiceGameListener::STOP;
	}
	++m_round;
	m_winner = 0;
	m_listener->startRound();
	m_dice[0]->shuffle();
	m_dice[1]->shuffle();
	m_winner = judge(result);
	if (m_winner == COMPUTER) {
		m_player->loseGame();
	} else {
		m_player->winGame();
	}
	m_listener->endRound();
	return 0;
}

int DiceGame::inquireOddOrEven()
{
	int result = 0;
	do {
		result = m_listener->inquireOddOrEven();
	} while (result != DiceGameListener::ODD
			&& result != DiceGameListener::EVEN
			&& result != DiceGameListener::STOP);
	return result;
}

int DiceGame::judge(int oddOrEven)
{
	int sum = 0;
	for (std::vector<Dice*>::const_iterator p = m_dice.begin();
			p != m_dice.end(); ++p) {
		sum += (*p)->getValue();
	}
	return ((sum + oddOrEven) % 2 == 0) ? PLAYER : COMPUTER;
}
