/*
 * DiceGameTest.cpp
 */

#include "DiceGameTest.h"
#include "dicegame/DiceGamePlayer.h"
#include "dicegame/DiceGame.h"
#include <cppunit/TestSuite.h>
#include <cppunit/TestCaller.h>
#include <cppunit/RepeatedTest.h>

USING_NAMESPACE_CPPUNIT

int DiceGameTest::TestListener::inquireOddOrEven()
{
	++inquired;
	return (inquired < 5) ? inquired : STOP;
}

void DiceGameTest::testCreate()
{
	DiceGame diceGame(0);
	TEST_ASSERT_EQUALS(0, diceGame.getRound());
	TEST_ASSERT_EQUALS(0, diceGame.getWinner());
	TEST_ASSERT(diceGame.getPlayer() != 0);
	TEST_ASSERT_EQUALS(2, diceGame.getDice().size());
	TEST_ASSERT(diceGame.getDice()[0] != 0);
	TEST_ASSERT(diceGame.getDice()[1] != 0);
}

void DiceGameTest::testInquireOddOrEven()
{
	TestListener listener;
	DiceGame diceGame(&listener);
	int result = diceGame.inquireOddOrEven();
	TEST_ASSERT_EQUALS(DiceGameListener::ODD, result);
	result = diceGame.inquireOddOrEven();
	TEST_ASSERT_EQUALS(DiceGameListener::EVEN, result);
	result = diceGame.inquireOddOrEven();
	TEST_ASSERT_EQUALS(DiceGameListener::STOP, result);
	result = diceGame.inquireOddOrEven();
	TEST_ASSERT_EQUALS(DiceGameListener::STOP, result);
}

void DiceGameTest::testJudge()
{
	DiceGame diceGame(0);
	int winner = diceGame.judge(DiceGameListener::ODD);
	TEST_ASSERT_EQUALS(DiceGame::COMPUTER, winner);
	winner = diceGame.judge(DiceGameListener::EVEN);
	TEST_ASSERT_EQUALS(DiceGame::PLAYER, winner);
}

void DiceGameTest::testDoRound()
{
	TestListener listener;
	DiceGame diceGame(&listener);
	TEST_ASSERT_EQUALS(0, listener.started);
	TEST_ASSERT_EQUALS(0, listener.ended);
	TEST_ASSERT_EQUALS(0, listener.inquired);
	diceGame.doRound();
	TEST_ASSERT_EQUALS(1, listener.started);
	TEST_ASSERT_EQUALS(1, listener.ended);
	TEST_ASSERT_EQUALS(1, listener.inquired);
	TEST_ASSERT_EQUALS(1, diceGame.getRound());
	int winner = diceGame.getWinner();
	DiceGamePlayer* player = diceGame.getPlayer();
	if (winner == DiceGame::COMPUTER) {
		TEST_ASSERT_EQUALS(0, player->getWins());
		TEST_ASSERT_EQUALS(1, player->getLosses());
	} else {
		TEST_ASSERT_EQUALS(DiceGame::PLAYER, winner);
		TEST_ASSERT_EQUALS(1, player->getWins());
		TEST_ASSERT_EQUALS(0, player->getLosses());
	}
}

void DiceGameTest::testRun()
{
	TestListener listener;
	DiceGame diceGame(&listener);
	TEST_ASSERT_EQUALS(0, listener.started);
	TEST_ASSERT_EQUALS(0, listener.ended);
	TEST_ASSERT_EQUALS(0, listener.inquired);
	diceGame.run();
	TEST_ASSERT_EQUALS(2, listener.started);
	TEST_ASSERT_EQUALS(2, listener.ended);
	TEST_ASSERT_EQUALS(5, listener.inquired);
	TEST_ASSERT_EQUALS(2, diceGame.getRound());
	DiceGamePlayer* player = diceGame.getPlayer();
	TEST_ASSERT_EQUALS(2, player->getWins() + player->getLosses());
}

Test* DiceGameTest::suite()
{
	TestSuite* suite = new TestSuite("DiceGameTest");
	suite->addTest(new TEST_CALLER(DiceGameTest, testCreate));
	suite->addTest(new TEST_CALLER(DiceGameTest, testInquireOddOrEven));
	suite->addTest(new TEST_CALLER(DiceGameTest, testJudge));
	suite->addTest(new RepeatedTest(
			new TEST_CALLER(DiceGameTest, testDoRound), 4));
	suite->addTest(new TEST_CALLER(DiceGameTest, testRun));
	return suite;
}
